part of masamune.flutter;

/// Abstract class of widget including listening function equivalent to UIValue
/// Inherit and use to create a widget with pass-list functionality
/// Please use StatelessWidget below this
/// In that case, the value can be obtained instantly by using [value.text()] or [value.get())].
abstract class UIWidget extends StatefulWidget {
  final PathEvent _load;
  final PathEvent _unload;
  final WidgetBuilder _child;
  final PathEvent _pause;
  final PathEvent _unpause;
  final PathEvent _quit;
  /// Abstract class of widget including listening function equivalent to UIValue
  /// Inherit and use to create a widget with pass-list functionality
  /// Please use StatelessWidget below this
  /// In that case, the value can be obtained instantly by using [value.text()] or [value.get())].
  /// 
  /// [load]: Callback for widget loading
  /// [unload]: Callback for Widget unloading
  /// [child]: Callback when creating a widget
  /// [pause]: Callback for Application paused
  /// [unpause]: Callback for Application unpaused
  /// [quit]: Callback for Application quit
  UIWidget({PathEvent load, PathEvent unload, PathEvent pause, PathEvent unpause, PathEvent quit, @required WidgetBuilder child}) : this._load = load, this._unload = unload, this._pause = pause, this._unpause = unpause, this._quit = quit, this._child = child;
  /// Create a state
  /// Do not use from outside
  @override
  @protected
  State createState() => _UIWidgetState(
    load: ( context, path ){ if( this._load != null ) this._load(context, path); this.onLoad(context, path); },
    unload: ( context, path ){ if( this._unload != null ) this._unload(context, path); this.onUnload(context, path); },
    pause: ( context, path ){ if( this._pause != null ) this._pause(context, path); this.onPause(context, path); },
    unpause: ( context, path ){ if( this._unpause != null ) this._unpause(context, path); this.onUnpause(context, path); },
    quit: ( context, path ){ if( this._quit != null ) this._quit(context, path); this.onQuit(context, path); },
    child: this._child ?? this.build
  );

  /// Callback for building
  /// Override and use
  /// 
  /// [context]: Build context
  @protected
  Widget build( BuildContext context ) => null;
  /// Executed when the widget is loaded
  /// Override and use
  /// 
  /// [context]: Build context
  /// [path]: Path list
  @protected
  onLoad( BuildContext context, PathList path ){ }
  /// Executed when the widget is unloaded
  /// Override and use
  /// 
  /// [context]: Build context
  /// [path]: Path list
  @protected
  onUnload( BuildContext context, PathList path ){ }
  /// Executed when the widget is pause
  /// Override and use
  /// 
  /// [context]: Build context
  /// [path]: Path list
  @protected
  onPause( BuildContext context, PathList path ){ }
  /// Executed when the widget is unpause
  /// Override and use
  /// 
  /// [context]: Build context
  /// [path]: Path list
  @protected
  onUnpause( BuildContext context, PathList path ){ }
  /// Executed when the widget is quit
  /// Override and use
  /// 
  /// [context]: Build context
  /// [path]: Path list
  @protected
  onQuit( BuildContext context, PathList path ) { }
}
class _UIWidgetState extends State<UIWidget> with WidgetsBindingObserver {
  final PathEvent _load;
  final PathEvent _unload;
  final WidgetBuilder _child;
  final PathEvent _pause;
  final PathEvent _unpause;
  final PathEvent _quit;
  UIValue _value;
  PathList _pathList = PathList();
  void _notifyUpdate() {
    Log.msg( "Rebuild widget: %s (%s)", [this.widget.runtimeType, this.widget.key?.toString()] );
    this.setState((){});
  }
  _UIWidgetState({PathEvent load, PathEvent unload, PathEvent pause, PathEvent unpause, PathEvent quit, @required WidgetBuilder child}) : this._load = load, this._unload = unload, this._pause = pause, this._unpause = unpause, this._quit = quit, this._child = child {
    this._value = UIValue._( this );
  }
  @override
  Widget build(BuildContext context) {
    return  this._child != null ? this._child( context ) : null;
  }
  @override
  void initState() {
    super.initState();
    _ContextMap.register(this.context, this);
    if( this._load != null ) this._load( this.context, this._pathList );
    if( this._pause != null || this._unpause != null || this._quit != null ) WidgetsBinding.instance.addObserver(this);
  }
  @override
  void dispose() {
    super.dispose();
    _ContextMap.unregister(this);
    if( this._unload != null ) this._unload( this.context, this._pathList );
    WidgetsBinding.instance.removeObserver(this);
    this._pathList.dispose();
    this._value._dispose();
  }
  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    switch(state){
      case AppLifecycleState.detached : if( this._quit != null ) this._quit( this.context, this._pathList ); break;
      case AppLifecycleState.paused : if( this._pause != null ) this._pause( this.context, this._pathList ); break;
      case AppLifecycleState.resumed : if( this._unpause != null ) this._unpause( this.context, this._pathList ); break;
      default: break;
    }
  }
}