part of masamune.flutter;

/// NavigatorState extension methods
extension NavigatorStateExtension on NavigatorState {
  /// Replace page.
  /// 
  /// [path]: Page path
  /// [document]: Document data to pass to the next page
  /// [documentPath]: Document data path to pass to the next page
  /// [routePath]: Page root path
  /// [pageDataPath]: Path to get passed data
  Future replacePath(
    String path,
    {
      IDataDocument document,
      String documentPath,
      String routePath = DefaultPath.route,
      String pageDataPath = DefaultPath.pageData
    }
  ){
    if( isEmpty(path) ) return null;
    path = path.trimString(Const.slash);
    DataField data = PathMap.get(Paths.child(routePath, path));
    if( data == null || !( data.data is WidgetBuilder ) ) return null;
    return this.pushReplacement(
      MaterialPageRoute(builder: data.data)
    );
  }
  /// Transition the page by specifying the path
  /// 
  /// [path]: Page path
  /// [document]: Document data to pass to the next page
  /// [documentPath]: Document data path to pass to the next page
  /// [routePath]: Page root path
  /// [pageDataPath]: Path to get passed data
  Future pushPath(
    String path,
    {
      IDataDocument document,
      String documentPath,
      String routePath = DefaultPath.route,
      String pageDataPath = DefaultPath.pageData
    }
  ){
    if( isEmpty(path) ) return null;
    path = path.trimString(Const.slash);
    DataField data = PathMap.get(Paths.child(routePath, path));
    if( data == null || !( data.data is WidgetBuilder ) ) return null;
    return this.push(
      SlidePageRoute(
        child: ( data.data as WidgetBuilder )( this.context ),
        document: document,
        documentPath: documentPath,
        pageDataPath: pageDataPath
      )
    );
  }
}