part of masamune.flutter;

/// Show dialog
/// ```
/// UIDialog.show( context );
/// ```
class UIDialog {
  /// Show dialog
  /// 
  /// [context]: Build context
  /// [dialogTitlePath]: Dialog title path
  /// [dialogTextPath]: Dialog text path
  /// [dialogSubmitTextPath]: Dialog submit button text path
  /// [dialogSubmitActionPath]: The path of action when the submit button of the dialog is pressed
  /// [dialogCancelTextPath]: Dialog cancel button text path
  /// [dialogCancelActionPath]: The path of action when the cancel button of the dialog is pressed
  /// [defaultSubmitText]: Default submit button text
  /// [defaultCancelText]: Default cancel button text
  /// [defaultSubmitAction]: Default submit button action
  /// [defaultCancelAction]: Default cancel button action
  /// [dialogTitle]: Default title
  /// [dialogText]: Default text
  static Future show(
    BuildContext context,
    {
      String dialogTitlePath = DefaultPath.dialogTitle,
      String dialogTextPath = DefaultPath.dialogText,
      String dialogSubmitTextPath = DefaultPath.dialogSubmitText,
      String dialogSubmitActionPath = DefaultPath.dialogSubmitAction,
      String defaultSubmitText = "OK",
      String defaultTitle,
      String defaultText,
      VoidAction defaultSubmitAction
    }
  ) async {
    if( context == null ) return;
    String title = context.text(dialogTitlePath, defaultValue: defaultTitle, listen: false);
    String text = context.text(dialogTextPath, defaultValue: defaultText, listen: false);
    if( title == null || text == null ) return;
    await showDialog(
      context: context,
      barrierDismissible: false,
      builder: (context) {
        return AlertDialog(
          title: Text(title),
          content: Text(text),
          actions: <Widget>[
            FlatButton(
              child: Text(context.text(dialogSubmitTextPath, defaultValue: defaultSubmitText, listen: false)),
              onPressed: () {
                PathMap.removeAllPath([dialogTitlePath, dialogTextPath, dialogSubmitTextPath, dialogSubmitActionPath, dialogSubmitTextPath]);
                Navigator.of(context, rootNavigator: true).pop();
                context.action(dialogSubmitActionPath, listen: false, defaultAction: defaultSubmitAction)();
              },
            )
          ],
        );
      }
    );
    PathMap.removeAllPath([dialogTitlePath, dialogTextPath, dialogSubmitTextPath, dialogSubmitActionPath, dialogSubmitTextPath]);
  }
}